REM install the code and begin the first time interval
Install(27)   '27 clock ticks of 0.838us each delay before starting ..


DO

PRINT timer :REM print timer just to show it is still running correctly

REM keep looping until a key is pressed
LOOP UNTIL instat


REM unistall the interrupt routine
UnInstall


END



SUB install(InitValue%)
REM call this sub to install the interrupt routine by intercepting the timer interrupt

low?=InitValue% MOD 256
high?=InitValue% \ 256

!cli			;disable interrupts while changing interrupt vector

!mov al,&h34            ;make sure PIT timer0 is in mode 2
!out &h43,al
!mov ax,low?            ;and set to initial timer value
!out &h40,al
!mov al,high?
!out &h40,al

!les bx,Int8V		;get address of Int8 vector
!mov ax,es:[bx]		;copy offset to ExitVector
!mov ExitVector[01],ax
!mov ax,es:[bx+02]	;copy segment to ExitVector
!mov ExitVector[03],ax

!mov ax,0
!mov TimeSoFar,ax	;clear count of ticks so far

!lea ax,Int8Entry       ;get int routine entry to Int8 vector
!mov es:[bx],ax         ;move offset of my routine to Int8 vector
!mov es:[bx+02],cs      ;move segment of my routine to Int8 vector
!sti                    ;Enable interrupts

END SUB



SUB uninstall
REM Call this SUB to restore the interrupt vectors when finished.

!cli			;disable interrupts while restoring interrupt vector
!les bx,Int8V		;get address of Int8 vector
!mov ax,ExitVector[01]  ;Restore Int8 vector
!mov es:[bx],ax		;restore offset
!mov ax,ExitVector[03]
!mov es:[bx+02],ax	;restore segment

!mov al,&h34            ;restore correct timer contents
!out &h43,al
!mov al,0
!out &h40,al
!out &h40,al

!sti                    ;Enable interrupts
END SUB



SUB introutine
REM this is the actual interrupt routine that will be called on each timer
REM interrupt.

REM Int8 is the hardware timer interrupt for the 18.2Hz clock

Int8Entry:
!pushf                  ;save flags
!push ax                ;save ax


!mov al,255		;pulse the output, use your own port number
!out 128,al             ;my ouput port is at 128
!mov al,0
!out 128,al


!mov ax,NextValue       ;update count of ticks so far
!add TimeSoFar,ax

!jnb DontChain          ;if count is <65536 then don't exit through old vector

Int8Exit:
!pop ax			;restore ax
!popf                   ;restore flags

ExitVector:
!jmp far ExitVector 	;exit via copy of original int8 vector copied into
                        'this location immediately after the JMP byte

DontChain:
!mov al,&h60		;clear pending int0 from timer chip myself
!out &h20,al

!pop ax			;restore ax
!popf                   ;restore flags
!IRET			;direct return from interrupt

Int8V:
!dd &h00000020		;Int8 vector location

NextValue:
!dd 27			;next value to be loaded into timer
			'rem the value to load into the PIT to give the
                        'required interrupt rate
          		'27 *0.838us/tick = 22.626us -> 44,192Hz

TimeSoFar:
!dd 0			;cummulative count of 0.838us ticks so far. When this
                        'exceeds 65536 then exit via the old Int8 vector to
			'cause a timer update, else exit using IRET.
END SUB
